subroutine continuity(ieq)
  use vartype
  use work
  use iface                     !DDZ function
  implicit none
  integer(i4b) :: ieq,iscratch,k
  real(sp) :: nbc(4)
  real(sp), dimension(:), pointer :: sw,nadv,uadv,flux,nho,nhn

  sw => scratch(:,1)
  nadv => scratch(:,2)
  uadv => scratch(:,3)
  flux => scratch(:,4)
  nho => scratch(:,5)
  nhn => scratch(:,6)
  iscratch = 6
  ! be absolutely certain that you don't go outside the allocated
  ! memory or else...
  if (iscratch > mscratch) stop 'continuity: iscratch>mscratch'
  
  e(ieq,:) = -v*(nn-no)         !time derivative dn/dt
  w(ieq,:,ieq,0) = w(ieq,:,ieq,0) + v !-dE/dnn

  sw = sign(half,u)             !+1/2 for u>0 and -1/2 for u<0
  nbc(1:2) = no(1)
  nbc(3:4) = no(ngrid)
  call fladv(no,nbc,dz,sw,.false.,ngrid,nho,dnhdn,dnhdz)
  nbc(1:2) = nn(1)
  nbc(3:4) = nn(ngrid)
  call fladv(nn,nbc,dz,sw,.true.,ngrid,nhn,dnhdn,dnhdz)
  nadv = beta*nhn + (1.-beta)*nho
  uadv = u                      !=beta*un+(1-beta)*uo when U changes in time
  flux = a*dt*nadv*uadv         !flux through surface A in time DT

  !then derivatives needed for W matrix when running implicitly (beta
  !>0):
  dfldn(:,-2) = a*dt*uadv*beta*dnhdn(:,-2) !d/dn_{k-2}
  dfldn(:,-1) = a*dt*uadv*beta*dnhdn(:,-1)
  dfldn(:,0)  = a*dt*uadv*beta*dnhdn(:,0)
  dfldn(:,1)  = a*dt*uadv*beta*dnhdn(:,1)
  dfldn(:,2)  = a*dt*uadv*beta*dnhdn(:,2)

  e(ieq,:) = e(ieq,:) - ddz(flux)
  do k=1,ngrid-1
     w(ieq,k,ieq,-2) = w(ieq,k,ieq,-2)-dfldn(k,-2) !d/dn[k-2]
     w(ieq,k,ieq,-1) = w(ieq,k,ieq,-1)+dfldn(k+1,-2)-dfldn(k,-1) !d/dn[k-1]
     w(ieq,k,ieq,0) = w(ieq,k,ieq,0)+dfldn(k+1,-1)-dfldn(k,0) !d/dn[k]
     w(ieq,k,ieq,1) = w(ieq,k,ieq,1)+dfldn(k+1,0)-dfldn(k,1) !d/dn[k+1]
     w(ieq,k,ieq,2) = w(ieq,k,ieq,2)+dfldn(k+1,1) !d/dn[k+2]
     ! when solving the momentum equation for the flow speed U we
     ! must take additional derivatives with respect to u[k] and u[k
     !+1] here
  end do

  ! and if we have production and loss of particles as well, that
  ! should be added here

  return
end subroutine continuity
