      subroutine pentag(nvar,ndep,nvstart,b,a)
      use vartype
c        NVAR: Number of equations that we solve (continuity and
c        momentum equation -> nvar=2).
c        NDEP: Number of altitude grid points.
c        NVSTART: Only equation numbers from NVSTART to NVAR are
c        actually solved for.  NVSTART=1 means all equations.
c        B: The array containing the errors for each equation and grid
c        point.
c        A: Derivatives of B with respect to all variables that we solve
c        for.

c        $Id: pentag.f,v 1.3 2003/08/14 14:31:35 ols Exp $
c        Solves system of equations Ax=b where a is assumed block
c        pentadiagonal.
c        140403 ols: NVSTART added.  Specifies the first equation that
c        we actually solve, so we don't waste time inverting the parts we'
c        re never going to use.
c        v. hansteen 30-5-91

      implicit none
      integer(i4b) :: nvar,ndep,idep,nvstart
      real(sp) b(nvar,ndep),a(nvar,ndep,nvar,-2:2)

      idep=1
      call diag(idep,nvar,ndep,nvstart,b,a)
      do 10 idep=2,ndep
        call red(idep,nvar,ndep,nvstart,b,a)
        call diag(idep,nvar,ndep,nvstart,b,a)
 10   continue
      call bksub(nvar,ndep,nvstart,b,a)

      return
      end

      subroutine diag(ib,nv,nb,nvstart,b,a)
      use vartype

c  Called by pentag. 
c  Diagonalize block ib
c                    nv   : number of varibles in block
c                    nb   : number of blocks
c                    b    : right hand side
c                    a    : grand matrix
c  This version assumes all diagonal elements != 0.
c  v. hansteen 30-5-91

      implicit none
      integer(i4b) :: ib,nv,nb,l,k,i,j,nvstart
      real(sp) b(nv,nb),a(nv,nb,nv,-2:2)
      real(sp) ainv,ajl

c   loop over columns
      do 200 l=nvstart,nv

c   normalize pivot row
        ainv=1.0/a(l,ib,l,0)
        do 130 k=0,2
        do 130 i=nvstart,nv
          a(l,ib,i,k)=a(l,ib,i,k)*ainv
 130    continue
        a(l,ib,l,0)=1.0
        b(l,ib)=b(l,ib)*ainv

c   zero column l if row j != l
        do 150 j=nvstart,nv
          ajl=a(j,ib,l,0)
          if(j.ne.l.and.ajl.ne.0.0)then
            do 140 k=0,2  
            do 140 i=nvstart,nv
              a(j,ib,i,k)=a(j,ib,i,k)-a(l,ib,i,k)*ajl
 140        continue
            a(j,ib,l,0)=0.0
            b(j,ib)=b(j,ib)-b(l,ib)*ajl
          endif
 150    continue
 200  continue

      return
      end

      subroutine red(ib,nv,nb,nvstart,b,a)
      use vartype

c  Called by pentag.
c  Reduce blocks under the diagonal to zero.
c                   ib    : block position
c                   nv    : number of variables in block
c                   nb    : number of blocks
c                   b     : right hand side
c                   a     : grand matrix
c  v. hansteen 31-5-91

      implicit none
      integer(i4b) :: ib,nv,nb,lb,kb,l,k,j,i,nvstart
      real(sp) b(nv,nb),a(nv,nb,nv,-2:2),ajl

c   loop over columns to be zeroed
      lb=-2
      if(ib.eq.2)lb=-1
      do 100 kb=lb,-1
      do 100 l=nvstart,nv

c   loop over columns to be altered
        do 10 k=1,2
        do 10 j=nvstart,nv
          ajl=a(l,ib+kb,j,k)

c   loop over rows
          do 10 i=nvstart,nv
            a(i,ib,j,k+kb)=a(i,ib,j,k+kb)-a(i,ib,l,kb)*ajl
 10     continue

c   right hand side
        do 20 i=nvstart,nv    
          b(i,ib)=b(i,ib)-b(l,ib+kb)*a(i,ib,l,kb)
 20     continue
 100  continue

      return
      end

      subroutine bksub(nv,nb,nvstart,b,a)
      use vartype

c  Called by pentag.
c  Back substitution after grand matrix reduced by diag and red.
c              nv   : number of variables in block
c              nb   : number of blocks
c              b    : right hand side, will contain answer
c              a    : grand matrix
c  v. hansteen 31-5-91

      implicit none
      integer(i4b) :: nv,nb,k,l,i,j,nvstart
      real(sp) b(nv,nb),a(nv,nb,nv,-2:2)
      real(sp) xi

c   ndep-1 block is special case
      k=nb-1
      l=1
      do 100 i=nv,nvstart,-1
        xi=b(i,k)
        do 10 j=nvstart,nv
          xi=xi-a(i,k,j,l)*b(j,k+l)
 10     continue
        b(i,k)=xi
 100  continue

c   ndep-2 to 1 block
      do 200 k=nb-2,1,-1
      do 200 i=nv,nvstart,-1
        xi=b(i,k)
        do 20 l=1,2
        do 20 j=nvstart,nv
          xi=xi-a(i,k,j,l)*b(j,k+l)
 20     continue
        b(i,k)=xi
 200  continue

      return
      end
